@@ -100,3 +100,10 @@ def sym(T):
 def dot(A, B):
 
     return _dot(transpose(A), B)
+
+
+def dev(T):
+
+    dim = T.shape[0]
+
+    return T - trace(T) / dim * eye(dim)
